# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options

__all__ = ["McpServersResource", "AsyncMcpServersResource"]


class McpServersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> McpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return McpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> McpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return McpServersResourceWithStreamingResponse(self)

    def create_mcp_server(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create a new MCP server for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/mcp-servers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete_mcp_server(
        self,
        mcp_server_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an MCP server from a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/missions/{mission_id}/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_mcp_server(
        self,
        mcp_server_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Get a specific MCP server by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list_mcp_servers(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        List all MCP servers for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/mcp-servers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update_mcp_server(
        self,
        mcp_server_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Update an MCP server definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return self._put(
            f"/ai/missions/{mission_id}/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncMcpServersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMcpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMcpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMcpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMcpServersResourceWithStreamingResponse(self)

    async def create_mcp_server(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create a new MCP server for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/mcp-servers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete_mcp_server(
        self,
        mcp_server_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an MCP server from a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/missions/{mission_id}/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_mcp_server(
        self,
        mcp_server_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Get a specific MCP server by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list_mcp_servers(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        List all MCP servers for a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/mcp-servers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update_mcp_server(
        self,
        mcp_server_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Update an MCP server definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return await self._put(
            f"/ai/missions/{mission_id}/mcp-servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class McpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create_mcp_server = to_raw_response_wrapper(
            mcp_servers.create_mcp_server,
        )
        self.delete_mcp_server = to_raw_response_wrapper(
            mcp_servers.delete_mcp_server,
        )
        self.get_mcp_server = to_raw_response_wrapper(
            mcp_servers.get_mcp_server,
        )
        self.list_mcp_servers = to_raw_response_wrapper(
            mcp_servers.list_mcp_servers,
        )
        self.update_mcp_server = to_raw_response_wrapper(
            mcp_servers.update_mcp_server,
        )


class AsyncMcpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create_mcp_server = async_to_raw_response_wrapper(
            mcp_servers.create_mcp_server,
        )
        self.delete_mcp_server = async_to_raw_response_wrapper(
            mcp_servers.delete_mcp_server,
        )
        self.get_mcp_server = async_to_raw_response_wrapper(
            mcp_servers.get_mcp_server,
        )
        self.list_mcp_servers = async_to_raw_response_wrapper(
            mcp_servers.list_mcp_servers,
        )
        self.update_mcp_server = async_to_raw_response_wrapper(
            mcp_servers.update_mcp_server,
        )


class McpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create_mcp_server = to_streamed_response_wrapper(
            mcp_servers.create_mcp_server,
        )
        self.delete_mcp_server = to_streamed_response_wrapper(
            mcp_servers.delete_mcp_server,
        )
        self.get_mcp_server = to_streamed_response_wrapper(
            mcp_servers.get_mcp_server,
        )
        self.list_mcp_servers = to_streamed_response_wrapper(
            mcp_servers.list_mcp_servers,
        )
        self.update_mcp_server = to_streamed_response_wrapper(
            mcp_servers.update_mcp_server,
        )


class AsyncMcpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create_mcp_server = async_to_streamed_response_wrapper(
            mcp_servers.create_mcp_server,
        )
        self.delete_mcp_server = async_to_streamed_response_wrapper(
            mcp_servers.delete_mcp_server,
        )
        self.get_mcp_server = async_to_streamed_response_wrapper(
            mcp_servers.get_mcp_server,
        )
        self.list_mcp_servers = async_to_streamed_response_wrapper(
            mcp_servers.list_mcp_servers,
        )
        self.update_mcp_server = async_to_streamed_response_wrapper(
            mcp_servers.update_mcp_server,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "TelephonySettings",
    "NoiseSuppressionConfig",
    "RecordingSettings",
    "VoicemailDetection",
    "VoicemailDetectionOnVoicemailDetected",
    "VoicemailDetectionOnVoicemailDetectedVoicemailMessage",
]


class NoiseSuppressionConfig(BaseModel):
    """Configuration for noise suppression.

    Only applicable when noise_suppression is 'deepfilternet'.
    """

    attenuation_limit: Optional[int] = None
    """Attenuation limit for noise suppression. Range: 0-100."""

    mode: Optional[Literal["advanced"]] = None
    """Mode for noise suppression configuration."""


class RecordingSettings(BaseModel):
    """Configuration for call recording format and channel settings."""

    channels: Optional[Literal["single", "dual"]] = None
    """The number of channels for the recording. 'single' for mono, 'dual' for stereo."""

    format: Optional[Literal["wav", "mp3"]] = None
    """The format of the recording file."""


class VoicemailDetectionOnVoicemailDetectedVoicemailMessage(BaseModel):
    """Configuration for the voicemail message to leave.

    Only applicable when action is 'leave_message_and_stop_assistant'.
    """

    message: Optional[str] = None
    """The specific message to leave as voicemail.

    Only applicable when type is 'message'.
    """

    prompt: Optional[str] = None
    """The prompt to use for generating the voicemail message.

    Only applicable when type is 'prompt'.
    """

    type: Optional[Literal["prompt", "message"]] = None
    """The type of voicemail message.

    Use 'prompt' to have the assistant generate a message based on a prompt, or
    'message' to leave a specific message.
    """


class VoicemailDetectionOnVoicemailDetected(BaseModel):
    """Action to take when voicemail is detected."""

    action: Optional[Literal["stop_assistant", "leave_message_and_stop_assistant", "continue_assistant"]] = None
    """The action to take when voicemail is detected."""

    voicemail_message: Optional[VoicemailDetectionOnVoicemailDetectedVoicemailMessage] = None
    """Configuration for the voicemail message to leave.

    Only applicable when action is 'leave_message_and_stop_assistant'.
    """


class VoicemailDetection(BaseModel):
    """
    Configuration for voicemail detection (AMD - Answering Machine Detection) on outgoing calls. These settings only apply if AMD is enabled on the Dial command. See [TeXML Dial documentation](https://developers.telnyx.com/api-reference/texml-rest-commands/initiate-an-outbound-call) for enabling AMD. Recommended settings: MachineDetection=Enable, AsyncAmd=true, DetectionMode=Premium.
    """

    on_voicemail_detected: Optional[VoicemailDetectionOnVoicemailDetected] = None
    """Action to take when voicemail is detected."""


class TelephonySettings(BaseModel):
    default_texml_app_id: Optional[str] = None
    """Default Texml App used for voice calls with your assistant.

    This will be created automatically on assistant creation.
    """

    noise_suppression: Optional[Literal["krisp", "deepfilternet", "disabled"]] = None
    """The noise suppression engine to use.

    Use 'disabled' to turn off noise suppression.
    """

    noise_suppression_config: Optional[NoiseSuppressionConfig] = None
    """Configuration for noise suppression.

    Only applicable when noise_suppression is 'deepfilternet'.
    """

    recording_settings: Optional[RecordingSettings] = None
    """Configuration for call recording format and channel settings."""

    supports_unauthenticated_web_calls: Optional[bool] = None
    """
    When enabled, allows users to interact with your AI assistant directly from your
    website without requiring authentication. This is required for FE widgets that
    work with assistants that have telephony enabled.
    """

    time_limit_secs: Optional[int] = None
    """Maximum duration in seconds for the AI assistant to participate on the call.

    When this limit is reached the assistant will be stopped. This limit does not
    apply to portions of a call without an active assistant (for instance, a call
    transferred to a human representative).
    """

    user_idle_timeout_secs: Optional[int] = None
    """Maximum duration in seconds of end user silence on the call.

    When this limit is reached the assistant will be stopped. This limit does not
    apply to portions of a call without an active assistant (for instance, a call
    transferred to a human representative).
    """

    voicemail_detection: Optional[VoicemailDetection] = None
    """
    Configuration for voicemail detection (AMD - Answering Machine Detection) on
    outgoing calls. These settings only apply if AMD is enabled on the Dial command.
    See
    [TeXML Dial documentation](https://developers.telnyx.com/api-reference/texml-rest-commands/initiate-an-outbound-call)
    for enabling AMD. Recommended settings: MachineDetection=Enable, AsyncAmd=true,
    DetectionMode=Premium.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ReportListWdrsResponse", "Cost", "DownlinkData", "Rate", "UplinkData"]


class Cost(BaseModel):
    amount: Optional[str] = None
    """Final cost. Cost is calculated as rate \\** unit"""

    currency: Optional[Literal["AUD", "CAD", "EUR", "GBP", "USD"]] = None
    """Currency of the rate and cost"""


class DownlinkData(BaseModel):
    amount: Optional[float] = None
    """Downlink data"""

    unit: Optional[Literal["B", "KB", "MB"]] = None
    """Transmission unit"""


class Rate(BaseModel):
    amount: Optional[str] = None
    """Rate from which cost is calculated"""

    currency: Optional[Literal["AUD", "CAD", "EUR", "GBP", "USD"]] = None
    """Currency of the rate and cost"""


class UplinkData(BaseModel):
    amount: Optional[float] = None
    """Uplink data"""

    unit: Optional[Literal["B", "KB", "MB"]] = None
    """Transmission unit"""


class ReportListWdrsResponse(BaseModel):
    id: Optional[str] = None
    """WDR id"""

    cost: Optional[Cost] = None

    created_at: Optional[datetime] = None
    """Record created time"""

    downlink_data: Optional[DownlinkData] = None

    duration_seconds: Optional[float] = None
    """Session duration in seconds."""

    imsi: Optional[str] = None
    """International mobile subscriber identity."""

    mcc: Optional[str] = None
    """Mobile country code."""

    mnc: Optional[str] = None
    """Mobile network code."""

    phone_number: Optional[str] = None
    """Phone number"""

    rate: Optional[Rate] = None

    record_type: Optional[str] = None

    sim_card_id: Optional[str] = None
    """Sim card unique identifier"""

    sim_group_id: Optional[str] = None
    """Sim group unique identifier"""

    sim_group_name: Optional[str] = None
    """Defined sim group name"""

    uplink_data: Optional[UplinkData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .brand_qualify_by_usecase_response import BrandQualifyByUsecaseResponse as BrandQualifyByUsecaseResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PhoneNumberAssignmentByProfileAssignResponse"]


class PhoneNumberAssignmentByProfileAssignResponse(BaseModel):
    messaging_profile_id: str = FieldInfo(alias="messagingProfileId")
    """
    The ID of the messaging profile that you want to link to the specified campaign.
    """

    task_id: str = FieldInfo(alias="taskId")
    """The ID of the task associated with assigning a messaging profile to a campaign."""

    campaign_id: Optional[str] = FieldInfo(alias="campaignId", default=None)
    """The ID of the campaign you want to link to the specified messaging profile.

    If you supply this ID in the request, do not also include a tcrCampaignId.
    """

    tcr_campaign_id: Optional[str] = FieldInfo(alias="tcrCampaignId", default=None)
    """
    The TCR ID of the shared campaign you want to link to the specified messaging
    profile (for campaigns not created using Telnyx 10DLC services only). If you
    supply this ID in the request, do not also include a campaignId.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.legacy.reporting.filter_param import FilterParam
from .....types.legacy.reporting.batch_detail_records import messaging_create_params
from .....types.legacy.reporting.batch_detail_records.messaging_list_response import MessagingListResponse
from .....types.legacy.reporting.batch_detail_records.messaging_create_response import MessagingCreateResponse
from .....types.legacy.reporting.batch_detail_records.messaging_delete_response import MessagingDeleteResponse
from .....types.legacy.reporting.batch_detail_records.messaging_retrieve_response import MessagingRetrieveResponse

__all__ = ["MessagingResource", "AsyncMessagingResource"]


class MessagingResource(SyncAPIResource):
    """Messaging batch detail records"""

    @cached_property
    def with_raw_response(self) -> MessagingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        end_time: Union[str, datetime],
        start_time: Union[str, datetime],
        connections: Iterable[int] | Omit = omit,
        directions: Iterable[int] | Omit = omit,
        filters: Iterable[FilterParam] | Omit = omit,
        include_message_body: bool | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        profiles: SequenceNotStr[str] | Omit = omit,
        record_types: Iterable[int] | Omit = omit,
        report_name: str | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingCreateResponse:
        """
        Creates a new MDR detailed report request with the specified filters

        Args:
          end_time: End time in ISO format. Note: If end time includes the last 4 hours, some MDRs
              might not appear in this report, due to wait time for downstream message
              delivery confirmation

          start_time: Start time in ISO format

          connections: List of connections to filter by

          directions: List of directions to filter by (Inbound = 1, Outbound = 2)

          filters: List of filters to apply

          include_message_body: Whether to include message body in the report

          managed_accounts: List of managed accounts to include

          profiles: List of messaging profile IDs to filter by

          record_types: List of record types to filter by (Complete = 1, Incomplete = 2, Errors = 3)

          report_name: Name of the report

          select_all_managed_accounts: Whether to select all managed accounts

          timezone: Timezone for the report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/legacy/reporting/batch_detail_records/messaging",
            body=maybe_transform(
                {
                    "end_time": end_time,
                    "start_time": start_time,
                    "connections": connections,
                    "directions": directions,
                    "filters": filters,
                    "include_message_body": include_message_body,
                    "managed_accounts": managed_accounts,
                    "profiles": profiles,
                    "record_types": record_types,
                    "report_name": report_name,
                    "select_all_managed_accounts": select_all_managed_accounts,
                    "timezone": timezone,
                },
                messaging_create_params.MessagingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingRetrieveResponse:
        """
        Retrieves a specific MDR detailed report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/legacy/reporting/batch_detail_records/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingListResponse:
        """Retrieves all MDR detailed report requests for the authenticated user"""
        return self._get(
            "/legacy/reporting/batch_detail_records/messaging",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingDeleteResponse:
        """
        Deletes a specific MDR detailed report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/legacy/reporting/batch_detail_records/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingDeleteResponse,
        )


class AsyncMessagingResource(AsyncAPIResource):
    """Messaging batch detail records"""

    @cached_property
    def with_raw_response(self) -> AsyncMessagingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        end_time: Union[str, datetime],
        start_time: Union[str, datetime],
        connections: Iterable[int] | Omit = omit,
        directions: Iterable[int] | Omit = omit,
        filters: Iterable[FilterParam] | Omit = omit,
        include_message_body: bool | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        profiles: SequenceNotStr[str] | Omit = omit,
        record_types: Iterable[int] | Omit = omit,
        report_name: str | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingCreateResponse:
        """
        Creates a new MDR detailed report request with the specified filters

        Args:
          end_time: End time in ISO format. Note: If end time includes the last 4 hours, some MDRs
              might not appear in this report, due to wait time for downstream message
              delivery confirmation

          start_time: Start time in ISO format

          connections: List of connections to filter by

          directions: List of directions to filter by (Inbound = 1, Outbound = 2)

          filters: List of filters to apply

          include_message_body: Whether to include message body in the report

          managed_accounts: List of managed accounts to include

          profiles: List of messaging profile IDs to filter by

          record_types: List of record types to filter by (Complete = 1, Incomplete = 2, Errors = 3)

          report_name: Name of the report

          select_all_managed_accounts: Whether to select all managed accounts

          timezone: Timezone for the report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/legacy/reporting/batch_detail_records/messaging",
            body=await async_maybe_transform(
                {
                    "end_time": end_time,
                    "start_time": start_time,
                    "connections": connections,
                    "directions": directions,
                    "filters": filters,
                    "include_message_body": include_message_body,
                    "managed_accounts": managed_accounts,
                    "profiles": profiles,
                    "record_types": record_types,
                    "report_name": report_name,
                    "select_all_managed_accounts": select_all_managed_accounts,
                    "timezone": timezone,
                },
                messaging_create_params.MessagingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingRetrieveResponse:
        """
        Retrieves a specific MDR detailed report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/legacy/reporting/batch_detail_records/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingListResponse:
        """Retrieves all MDR detailed report requests for the authenticated user"""
        return await self._get(
            "/legacy/reporting/batch_detail_records/messaging",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingDeleteResponse:
        """
        Deletes a specific MDR detailed report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/legacy/reporting/batch_detail_records/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingDeleteResponse,
        )


class MessagingResourceWithRawResponse:
    def __init__(self, messaging: MessagingResource) -> None:
        self._messaging = messaging

        self.create = to_raw_response_wrapper(
            messaging.create,
        )
        self.retrieve = to_raw_response_wrapper(
            messaging.retrieve,
        )
        self.list = to_raw_response_wrapper(
            messaging.list,
        )
        self.delete = to_raw_response_wrapper(
            messaging.delete,
        )


class AsyncMessagingResourceWithRawResponse:
    def __init__(self, messaging: AsyncMessagingResource) -> None:
        self._messaging = messaging

        self.create = async_to_raw_response_wrapper(
            messaging.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            messaging.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            messaging.list,
        )
        self.delete = async_to_raw_response_wrapper(
            messaging.delete,
        )


class MessagingResourceWithStreamingResponse:
    def __init__(self, messaging: MessagingResource) -> None:
        self._messaging = messaging

        self.create = to_streamed_response_wrapper(
            messaging.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            messaging.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            messaging.list,
        )
        self.delete = to_streamed_response_wrapper(
            messaging.delete,
        )


class AsyncMessagingResourceWithStreamingResponse:
    def __init__(self, messaging: AsyncMessagingResource) -> None:
        self._messaging = messaging

        self.create = async_to_streamed_response_wrapper(
            messaging.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            messaging.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            messaging.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            messaging.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["WhatsappCallingSettingsData"]


class WhatsappCallingSettingsData(BaseModel):
    enabled: Optional[bool] = None
    """True if calling is enabled on the phone"""

    phone_number: Optional[str] = None
    """Phone number in E164 format"""

    record_type: Optional[str] = None

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ChannelZoneListResponse"]


class ChannelZoneListResponse(BaseModel):
    id: str

    channels: int

    countries: List[str]
    """
    List of countries (in ISO 3166-2, capitalized) members of the billing channel
    zone
    """

    name: str

    record_type: Literal["channel_zone"]

    created_at: Optional[str] = None
    """ISO 8601 formatted date of when the channel zone was created"""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the channel zone was updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .inexplicit_number_order_response import InexplicitNumberOrderResponse

__all__ = ["InexplicitNumberOrderRetrieveResponse"]


class InexplicitNumberOrderRetrieveResponse(BaseModel):
    data: Optional[InexplicitNumberOrderResponse] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["HangupToolParamsParam"]


class HangupToolParamsParam(TypedDict, total=False):
    description: str
    """The description of the function that will be passed to the assistant."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.verifications.by_phone_number_list_response import ByPhoneNumberListResponse

__all__ = ["ByPhoneNumberResource", "AsyncByPhoneNumberResource"]


class ByPhoneNumberResource(SyncAPIResource):
    """Two factor authentication API"""

    @cached_property
    def actions(self) -> ActionsResource:
        """Two factor authentication API"""
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ByPhoneNumberResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ByPhoneNumberResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ByPhoneNumberResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ByPhoneNumberResourceWithStreamingResponse(self)

    def list(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ByPhoneNumberListResponse:
        """
        List verifications by phone number

        Args:
          phone_number: +E164 formatted phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/verifications/by_phone_number/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ByPhoneNumberListResponse,
        )


class AsyncByPhoneNumberResource(AsyncAPIResource):
    """Two factor authentication API"""

    @cached_property
    def actions(self) -> AsyncActionsResource:
        """Two factor authentication API"""
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncByPhoneNumberResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncByPhoneNumberResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncByPhoneNumberResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncByPhoneNumberResourceWithStreamingResponse(self)

    async def list(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ByPhoneNumberListResponse:
        """
        List verifications by phone number

        Args:
          phone_number: +E164 formatted phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/verifications/by_phone_number/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ByPhoneNumberListResponse,
        )


class ByPhoneNumberResourceWithRawResponse:
    def __init__(self, by_phone_number: ByPhoneNumberResource) -> None:
        self._by_phone_number = by_phone_number

        self.list = to_raw_response_wrapper(
            by_phone_number.list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        """Two factor authentication API"""
        return ActionsResourceWithRawResponse(self._by_phone_number.actions)


class AsyncByPhoneNumberResourceWithRawResponse:
    def __init__(self, by_phone_number: AsyncByPhoneNumberResource) -> None:
        self._by_phone_number = by_phone_number

        self.list = async_to_raw_response_wrapper(
            by_phone_number.list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        """Two factor authentication API"""
        return AsyncActionsResourceWithRawResponse(self._by_phone_number.actions)


class ByPhoneNumberResourceWithStreamingResponse:
    def __init__(self, by_phone_number: ByPhoneNumberResource) -> None:
        self._by_phone_number = by_phone_number

        self.list = to_streamed_response_wrapper(
            by_phone_number.list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        """Two factor authentication API"""
        return ActionsResourceWithStreamingResponse(self._by_phone_number.actions)


class AsyncByPhoneNumberResourceWithStreamingResponse:
    def __init__(self, by_phone_number: AsyncByPhoneNumberResource) -> None:
        self._by_phone_number = by_phone_number

        self.list = async_to_streamed_response_wrapper(
            by_phone_number.list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        """Two factor authentication API"""
        return AsyncActionsResourceWithStreamingResponse(self._by_phone_number.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.whatsapp.business_accounts import setting_update_params
from ....types.whatsapp.business_accounts.setting_update_response import SettingUpdateResponse
from ....types.whatsapp.business_accounts.setting_retrieve_response import SettingRetrieveResponse

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    """Manage Whatsapp business accounts"""

    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingRetrieveResponse:
        """
        Get WABA settings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v2/whatsapp/business_accounts/{id}/settings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        name: str | Omit = omit,
        timezone: str | Omit = omit,
        webhook_enabled: bool | Omit = omit,
        webhook_events: SequenceNotStr[str] | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingUpdateResponse:
        """
        Update WABA settings

        Args:
          timezone: IANA timezone identifier

          webhook_enabled: Enable/disable receiving Whatsapp events

          webhook_failover_url: Failover URL to send Whatsapp events

          webhook_url: URL to send Whatsapp events

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/v2/whatsapp/business_accounts/{id}/settings",
            body=maybe_transform(
                {
                    "name": name,
                    "timezone": timezone,
                    "webhook_enabled": webhook_enabled,
                    "webhook_events": webhook_events,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                setting_update_params.SettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingUpdateResponse,
        )


class AsyncSettingsResource(AsyncAPIResource):
    """Manage Whatsapp business accounts"""

    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingRetrieveResponse:
        """
        Get WABA settings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v2/whatsapp/business_accounts/{id}/settings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        name: str | Omit = omit,
        timezone: str | Omit = omit,
        webhook_enabled: bool | Omit = omit,
        webhook_events: SequenceNotStr[str] | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingUpdateResponse:
        """
        Update WABA settings

        Args:
          timezone: IANA timezone identifier

          webhook_enabled: Enable/disable receiving Whatsapp events

          webhook_failover_url: Failover URL to send Whatsapp events

          webhook_url: URL to send Whatsapp events

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/v2/whatsapp/business_accounts/{id}/settings",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "timezone": timezone,
                    "webhook_enabled": webhook_enabled,
                    "webhook_events": webhook_events,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                setting_update_params.SettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingUpdateResponse,
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.retrieve = to_raw_response_wrapper(
            settings.retrieve,
        )
        self.update = to_raw_response_wrapper(
            settings.update,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.retrieve = async_to_raw_response_wrapper(
            settings.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            settings.update,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.retrieve = to_streamed_response_wrapper(
            settings.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            settings.update,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.retrieve = async_to_streamed_response_wrapper(
            settings.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            settings.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .cloudflare_sync_status import CloudflareSyncStatus

__all__ = ["AccessIPRange"]


class AccessIPRange(BaseModel):
    id: str

    cidr_block: str

    status: CloudflareSyncStatus
    """An enumeration."""

    user_id: str

    created_at: Optional[datetime] = None

    description: Optional[str] = None

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["JobListParams", "Filter"]


class JobListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style). Originally: filter[type]"""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Literal["created_at"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order.
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style). Originally: filter[type]"""

    type: Literal["update_emergency_settings", "delete_phone_numbers", "update_phone_numbers"]
    """Identifies the type of the background job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .whatsapp_media import WhatsappMedia
from .whatsapp_contact import WhatsappContact
from .whatsapp_location import WhatsappLocation
from .whatsapp_reaction import WhatsappReaction
from .whatsapp_interactive import WhatsappInteractive

__all__ = ["WhatsappMessageContent"]


class WhatsappMessageContent(BaseModel):
    audio: Optional[WhatsappMedia] = None

    biz_opaque_callback_data: Optional[str] = None
    """custom data to return with status update"""

    contacts: Optional[List[WhatsappContact]] = None

    document: Optional[WhatsappMedia] = None

    image: Optional[WhatsappMedia] = None

    interactive: Optional[WhatsappInteractive] = None

    location: Optional[WhatsappLocation] = None

    reaction: Optional[WhatsappReaction] = None

    sticker: Optional[WhatsappMedia] = None

    type: Optional[
        Literal[
            "audio",
            "document",
            "image",
            "sticker",
            "video",
            "interactive",
            "location",
            "template",
            "reaction",
            "contacts",
        ]
    ] = None

    video: Optional[WhatsappMedia] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SimCardGetActivationCodeResponse", "Data"]


class Data(BaseModel):
    activation_code: Optional[str] = None
    """Contents of the eSIM activation QR code."""

    record_type: Optional[str] = None


class SimCardGetActivationCodeResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import TypedDict

__all__ = ["WhatsappContactParam", "Address", "Email", "Org", "Phone", "URL"]


class Address(TypedDict, total=False):
    city: str

    country: str

    country_code: str

    state: str

    street: str

    type: str

    zip: str


class Email(TypedDict, total=False):
    email: str

    type: str


class Org(TypedDict, total=False):
    company: str

    department: str

    title: str


class Phone(TypedDict, total=False):
    phone: str

    type: str

    wa_id: str


class URL(TypedDict, total=False):
    type: str

    url: str


class WhatsappContactParam(TypedDict, total=False):
    addresses: Iterable[Address]

    birthday: str

    emails: Iterable[Email]

    name: str

    org: Org

    phones: Iterable[Phone]

    urls: Iterable[URL]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_gather_ended import CallGatherEnded

__all__ = ["CallGatherEndedWebhookEvent"]


class CallGatherEndedWebhookEvent(BaseModel):
    data: Optional[CallGatherEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import dynamic_emergency_address_list_params, dynamic_emergency_address_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.dynamic_emergency_address import DynamicEmergencyAddress
from ..types.dynamic_emergency_address_create_response import DynamicEmergencyAddressCreateResponse
from ..types.dynamic_emergency_address_delete_response import DynamicEmergencyAddressDeleteResponse
from ..types.dynamic_emergency_address_retrieve_response import DynamicEmergencyAddressRetrieveResponse

__all__ = ["DynamicEmergencyAddressesResource", "AsyncDynamicEmergencyAddressesResource"]


class DynamicEmergencyAddressesResource(SyncAPIResource):
    """Dynamic emergency address operations"""

    @cached_property
    def with_raw_response(self) -> DynamicEmergencyAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return DynamicEmergencyAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DynamicEmergencyAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return DynamicEmergencyAddressesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        administrative_area: str,
        country_code: Literal["US", "CA", "PR"],
        house_number: str,
        locality: str,
        postal_code: str,
        street_name: str,
        extended_address: str | Omit = omit,
        house_suffix: str | Omit = omit,
        street_post_directional: str | Omit = omit,
        street_pre_directional: str | Omit = omit,
        street_suffix: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyAddressCreateResponse:
        """
        Creates a dynamic emergency address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/dynamic_emergency_addresses",
            body=maybe_transform(
                {
                    "administrative_area": administrative_area,
                    "country_code": country_code,
                    "house_number": house_number,
                    "locality": locality,
                    "postal_code": postal_code,
                    "street_name": street_name,
                    "extended_address": extended_address,
                    "house_suffix": house_suffix,
                    "street_post_directional": street_post_directional,
                    "street_pre_directional": street_pre_directional,
                    "street_suffix": street_suffix,
                },
                dynamic_emergency_address_create_params.DynamicEmergencyAddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyAddressCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyAddressRetrieveResponse:
        """
        Returns the dynamic emergency address based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/dynamic_emergency_addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyAddressRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: dynamic_emergency_address_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[DynamicEmergencyAddress]:
        """
        Returns the dynamic emergency addresses according to filters

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[country_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/dynamic_emergency_addresses",
            page=SyncDefaultFlatPagination[DynamicEmergencyAddress],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    dynamic_emergency_address_list_params.DynamicEmergencyAddressListParams,
                ),
            ),
            model=DynamicEmergencyAddress,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyAddressDeleteResponse:
        """
        Deletes the dynamic emergency address based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/dynamic_emergency_addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyAddressDeleteResponse,
        )


class AsyncDynamicEmergencyAddressesResource(AsyncAPIResource):
    """Dynamic emergency address operations"""

    @cached_property
    def with_raw_response(self) -> AsyncDynamicEmergencyAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDynamicEmergencyAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDynamicEmergencyAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncDynamicEmergencyAddressesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        administrative_area: str,
        country_code: Literal["US", "CA", "PR"],
        house_number: str,
        locality: str,
        postal_code: str,
        street_name: str,
        extended_address: str | Omit = omit,
        house_suffix: str | Omit = omit,
        street_post_directional: str | Omit = omit,
        street_pre_directional: str | Omit = omit,
        street_suffix: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyAddressCreateResponse:
        """
        Creates a dynamic emergency address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/dynamic_emergency_addresses",
            body=await async_maybe_transform(
                {
                    "administrative_area": administrative_area,
                    "country_code": country_code,
                    "house_number": house_number,
                    "locality": locality,
                    "postal_code": postal_code,
                    "street_name": street_name,
                    "extended_address": extended_address,
                    "house_suffix": house_suffix,
                    "street_post_directional": street_post_directional,
                    "street_pre_directional": street_pre_directional,
                    "street_suffix": street_suffix,
                },
                dynamic_emergency_address_create_params.DynamicEmergencyAddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyAddressCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyAddressRetrieveResponse:
        """
        Returns the dynamic emergency address based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/dynamic_emergency_addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyAddressRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: dynamic_emergency_address_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DynamicEmergencyAddress, AsyncDefaultFlatPagination[DynamicEmergencyAddress]]:
        """
        Returns the dynamic emergency addresses according to filters

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[country_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/dynamic_emergency_addresses",
            page=AsyncDefaultFlatPagination[DynamicEmergencyAddress],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    dynamic_emergency_address_list_params.DynamicEmergencyAddressListParams,
                ),
            ),
            model=DynamicEmergencyAddress,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyAddressDeleteResponse:
        """
        Deletes the dynamic emergency address based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/dynamic_emergency_addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyAddressDeleteResponse,
        )


class DynamicEmergencyAddressesResourceWithRawResponse:
    def __init__(self, dynamic_emergency_addresses: DynamicEmergencyAddressesResource) -> None:
        self._dynamic_emergency_addresses = dynamic_emergency_addresses

        self.create = to_raw_response_wrapper(
            dynamic_emergency_addresses.create,
        )
        self.retrieve = to_raw_response_wrapper(
            dynamic_emergency_addresses.retrieve,
        )
        self.list = to_raw_response_wrapper(
            dynamic_emergency_addresses.list,
        )
        self.delete = to_raw_response_wrapper(
            dynamic_emergency_addresses.delete,
        )


class AsyncDynamicEmergencyAddressesResourceWithRawResponse:
    def __init__(self, dynamic_emergency_addresses: AsyncDynamicEmergencyAddressesResource) -> None:
        self._dynamic_emergency_addresses = dynamic_emergency_addresses

        self.create = async_to_raw_response_wrapper(
            dynamic_emergency_addresses.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            dynamic_emergency_addresses.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            dynamic_emergency_addresses.list,
        )
        self.delete = async_to_raw_response_wrapper(
            dynamic_emergency_addresses.delete,
        )


class DynamicEmergencyAddressesResourceWithStreamingResponse:
    def __init__(self, dynamic_emergency_addresses: DynamicEmergencyAddressesResource) -> None:
        self._dynamic_emergency_addresses = dynamic_emergency_addresses

        self.create = to_streamed_response_wrapper(
            dynamic_emergency_addresses.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            dynamic_emergency_addresses.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            dynamic_emergency_addresses.list,
        )
        self.delete = to_streamed_response_wrapper(
            dynamic_emergency_addresses.delete,
        )


class AsyncDynamicEmergencyAddressesResourceWithStreamingResponse:
    def __init__(self, dynamic_emergency_addresses: AsyncDynamicEmergencyAddressesResource) -> None:
        self._dynamic_emergency_addresses = dynamic_emergency_addresses

        self.create = async_to_streamed_response_wrapper(
            dynamic_emergency_addresses.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            dynamic_emergency_addresses.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            dynamic_emergency_addresses.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            dynamic_emergency_addresses.delete,
        )
